/*
 *	LCD interface example
 *	Uses routines from delay.c
 *	This code will interface to a standard LCD controller
 *	like the Hitachi HD44780. It uses it in 4 or 8 bit mode
 *	
 */

#include	<pic.h>
#include	"lcd.h"
#include	"delay.h"
 

void
lcd_send(unsigned char c)
{
	LCD_EN = 1;
	LCD_TX(c>>4);	// load upper nibble
	LCD_EN = 0;
	NOP();
	NOP();
	LCD_EN = 1;
	LCD_TX(c);	// load lower nibble
	LCD_EN = 0;
}

/* send a command to the LCD */
void
lcd_cmd(unsigned char c)
{
	LCD_WAIT; // may check LCD busy flag, or just delay a little, depending on lcd.h
	LCD_RS = 0;
	lcd_send(c);
}

/* send data to the LCD */
void
lcd_data(unsigned char c)
{
	LCD_WAIT; // may check LCD busy flag, or just delay a little, depending on lcd.h

	LCD_RS = 1;
	lcd_send(c);
	LCD_RS = 0;
}

/* write a string of chars to the LCD */
void
lcd_puts(const char * s)
{
	while(*s)
		lcd_data(*s++);
}

/* initialize the LCD - 4 bit mode */
void
lcd_init() {
	char init_value;

	init_value = 0x3;

#if (_PICDEM2_REVISION_ == 6)
	LCD_CTRL &= 0x0F;
	// All data and control lines on same TRIS register
	LCD_DATA_TRIS	 &= (OUTPUT_DATA & OUTPUT_CTRL);
	LCD_VCC = 1;	// turn on the power to the LCD
#else
	LCD_RS = 0;
	LCD_EN = 0;
	LCD_RW = 0;
// Chips that disable analog functionality via ANSEL register should
// execute the following segment.
#if defined(__PIC16F887_H)
	// Set control lines to digital
	LCD_RS_ANSEL = 0;
	LCD_EN_ANSEL = 0;
	LCD_RW_ANSEL = 0;
#endif
	// Set control lines to output
	LCD_RS_TRIS	 = OUTPUT_PIN;
	LCD_EN_TRIS	 = OUTPUT_PIN;
	LCD_RW_TRIS	 = OUTPUT_PIN;
	// Set data lines to output
	LCD_DATA_TRIS	 &= OUTPUT_DATA;
#endif

	DelayMs(15);
	LCD_EN = 1;
	LCD_TX(init_value);
	LCD_EN = 0;
	DelayMs(5);
	LCD_EN = 1;
	LCD_DATA |= init_value;
	LCD_EN = 0;
	DelayUs(200);
	LCD_EN = 1;
	LCD_DATA |= init_value;
	LCD_EN = 0;
	DelayUs(200);
	
	LCD_WAIT;	//may check LCD busy flag, or just delay a little, depending on lcd.h
	LCD_EN = 1;
	LCD_TX(0x2);	// Set 4-bit mode
	LCD_EN = 0;
	lcd_cmd(0x28);	// Function Set

	lcd_cmd(0xF); //Display On, Cursor On, Cursor Blink
	lcd_clear(); //Display Clear
	lcd_cmd(0x6); //Entry Mode
	lcd_goto(0); //Initialize DDRAM address to zero
}

