/* File : fixed.c
 * Main Internal program as part of example for
 * splitting code on the PIC midrange chips
 */

#include <pic.h>
#include <string.h>

int inc(int);
int twice(int);
int dosomething(int);

int gl_fixed;

void
main(void)
{
	int lo_fixed;
	char buf[20];

	// Enable interrupts
	T0CS = 1;       // Set clock source to internal
	PEIE = 1;       // Enable peripheral interrupts
	T0IE = 1;       
	ei();   

	strcpy(buf, "hello");
	lo_fixed = strlen(buf);
	gl_fixed = twice(lo_fixed);
	
	// call external ext_main routine 
	(*((void(*)(int, char[]))0x4100))(gl_fixed, buf);
}

int
inc(int a)
{
	return a+1;
}

int
twice(int a)
{
	return a*2;
}

int
dosomething(int a)
{
	return twice(inc(a));
}

void interrupt
f_isr08(void) @ 0x8
{
	/* do whatever - code does not change as it is part
	 * of the internal program
	 */
}


/* For interrupts that require code updates, the internal 
 * interrupt will call an external function  ext_isr10 to do
 * all of the required work. To enable future changes, 
 * external function is defined as a psect labeledisr10 
 * and always located at 4200h. The internal interrupt f_isr10
 * always calls this address.
 */
void interrupt
f_isr10(void) @ 0x10
{
	// call the external function at 4200h
	((void(*)(void))0x4200)();
}

