/* File : r_ext.c
 * Replaceable External file as part of example
 * for splitting code on the PIC midrange chips
 */

int
half(int a)
{
	return (int)a/2;
}

int 
multiply(int a, int b)
{
	return a*b;
}

/* ext_isr10 is a function called by internal interrupt,
 * f_isr10, to do all the required work for f_isr10. 
 * Function ext_isr10 is declared as a psect labeled isr10, 
 * and placed at address 4200h. The internal interrupt 
 * f_isr10 always calls this address.
 */
#pragma psect text%%u=isr10

/*  When linking declare psect isr10 at address 4200, i.e.
	-pisr10=4200h
*/


/* As the internal interrupt f_isr10 calls the address 4200
 * and not the function ext_isr10, there is no function elsewhere
 * calling ext_isr10, and hence the linker may overlap the ext_isr10
 * function's auto and parameter areas with those of another
 * function. The linker needs to be informed that the ext_isr10
 * function is the "root" of a callgraph and memory be allocated 
 * for its auto and parameter areas.
 */
asm("FNROOT _ext_isr10");

void 
ext_isr10(void)
{
	// do whatever
	int a=6;
	a = a/2;
}

