
/*--------------------------------------------------------------
 *
 * Name:        FREQ.C
 *
 * Purpose:     Set display and frequency of signal generator.
 *
 * Inputs:      Command line.   See usage() for information.
 *
 * Outputs:     RS-232 port.
 *
 * History:     Original, John Langner, WB2OSZ.
 *
 *--------------------------------------------------------------*/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <bios.h>
#include <math.h>


#include "sg_cmd.h"


void usage();



int main (int argc, char *argv[])
{
        FILE *fp;

/*
 * From the command line.
 */
        int com_port_num;               /* For communicating with device. */
        double freq;
        char digits[20];

        int n;
        char port_name[10];

/*
 * Get the items from the command line.
 */
        com_port_num = 1;

        n = 1;

        if (n >= argc) {
            usage();
        }

        if (strnicmp(argv[n], "/com=", 5) == 0) {
            com_port_num = atoi(argv[n]+5);
            printf ("Using COM port %d.\n", com_port_num);
            n++;
        }

        if (n >= argc) {
            fprintf (stderr, "Frequency is missing.\n\n");
            usage();
        }

        freq = atof (argv[n]);
        n++;


        if (n < argc) {
            fprintf (stderr, "Too many command line arguments.\n\n");
            usage ();
        }

/*
 * Enforce some limits.
 */
        if (freq < 0)
            freq = 0;

        if (freq > 20e6)
            freq = 20e6;

/*
 * Convert to 9 characters.
 */
        sprintf (digits, "%09ld", (long)((freq * 10.) + .5));


/*
 * Send it out serial port.
 * This is a one way message; no ack comes back.
 *
 * _bios_serialcom is for Borland C.
 * Might not work for other compilers.
 */

        _bios_serialcom (_COM_INIT,
                com_port_num - 1,               /* 0 for COM1. */
                _COM_CHR8 | _COM_STOP1 | _COM_NOPARITY | _COM_300);

        sprintf (port_name, "COM%d", com_port_num);

        fp = fopen (port_name, "wb");
        if (fp == NULL) {
            fprintf (stderr, "ERROR: Couldn't open device %s.\n", port_name);
            exit (1);
        }

        fputc (FREQUENCY_CMD, fp);

        fwrite (digits, DIGITS, 1, fp);

        fclose (fp);

        return (0);     /* Quiet compiler warning. */
}



/*-----------------------------------------------------------------
 *
 * Name:        usage
 *
 * Purpose:     Print brief usage message and exit.
 *
 *----------------------------------------------------------------*/

void usage ()
{
        fprintf (stderr, "usage: \n");
        fprintf (stderr, "        FREQ [ /COM=n ] \"xxx\"\n");
        fprintf (stderr, "where,\n");
        fprintf (stderr, "        n         is optional COM port number.  Default 1.\n");
        fprintf (stderr, "        xxx       is freqency. 0 - 20 MHz, 0.1 Hz resolution.\n");
        fprintf (stderr, "\n");

        exit (1);

} /* end usage */

/* end FREQ.C */
