
/*--------------------------------------------------------------
 *
 * Name:        MESSAGE.C
 *
 * Purpose:     Put a message on display of signal generator.
 *
 * Inputs:      Command line.   See usage() for information.
 *
 * Outputs:     RS-232 port.
 *
 * History:     Original, John Langner, WB2OSZ.
 *
 *--------------------------------------------------------------*/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <bios.h>


#include "sg_cmd.h"


void usage();



int main (int argc, char *argv[])
{
        FILE *fp;

/*
 * From the command line.
 */
        int com_port_num;               /* For communicating with device. */
        int start_position;             /* Starting position, 0 = left. */
        char msg[80];                   /* Characters to display. */

        int n;
        char port_name[10];
        struct message_cmd_s cmd;

/*
 * Get the items from the command line.
 */
        com_port_num = 1;
        start_position = 0;

        n = 1;

        if (n >= argc) {
            usage();
        }

        if (strnicmp(argv[n], "/com=", 5) == 0) {
            com_port_num = atoi(argv[n]+5);
            printf ("Using COM port %d.\n", com_port_num);
            n++;
        }

        if (strnicmp(argv[n], "/start=", 7) == 0) {
            start_position = atoi(argv[n]+7);
            n++;
        }


        if (n >= argc) {
            fprintf (stderr, "Message text frequency is missing.\n\n");
            usage();
        }

        strcpy (msg, argv[n]);
        n++;


        if (n < argc) {
            fprintf (stderr, "Message text must be quoted if it contains blanks or special characters.\n\n");
            usage ();
        }

/*
 * Quietly enforce some limits.
 */
        if (start_position < 0)
            start_position = 0;

        if (start_position > 15)
            start_position = 15;

        msg[16 - start_position] = '\0';

/*
 * Fill in the command parameters.
 */
        cmd.position = start_position;
        cmd.len = strlen(msg);
        cmd.reserved = 0;

/*
 * Send it out serial port.
 * This is a one way message; no ack comes back.
 *
 * _bios_serialcom is for Borland C.
 * Might not work for other compilers.
 */

        _bios_serialcom (_COM_INIT,
                com_port_num - 1,               /* 0 for COM1. */
                _COM_CHR8 | _COM_STOP1 | _COM_NOPARITY | _COM_300);

        sprintf (port_name, "COM%d", com_port_num);

        fp = fopen (port_name, "wb");
        if (fp == NULL) {
            fprintf (stderr, "ERROR: Couldn't open device %s.\n", port_name);
            exit (1);
        }

        fputc (MESSAGE_CMD, fp);

        fwrite (&cmd, sizeof(cmd), 1, fp);
        fwrite (msg, strlen(msg), 1, fp);

        fclose (fp);

        return (0);     /* Quiet compiler warning. */
}



/*-----------------------------------------------------------------
 *
 * Name:        usage
 *
 * Purpose:     Print brief usage message and exit.
 *
 *----------------------------------------------------------------*/

void usage ()
{
        fprintf (stderr, "usage: \n");
        fprintf (stderr, "        MESSAGE [ /COM=n ] [ /START=s ] \"xxx\"\n");
        fprintf (stderr, "where,\n");
        fprintf (stderr, "        n         is optional COM port number.  Default 1.\n");
        fprintf (stderr, "        s         is starting position: 0 (left) to 15.\n");
        fprintf (stderr, "        xxx       is message.  Must be quoted if it contains blanks.\n");
        fprintf (stderr, "\n");

        exit (1);

} /* end usage */

/* end MESSAGE.C */
